#pragma once

#include <libdigidoc/DigiDocConfig.h>
#include <libdigidoc/DigiDocGen.h>

#include "BDocException.h"
#include "Document.h"
#include "Signature.h"

#ifdef _WIN32
#include <Windows.h>
#else
#include <dlfcn.h>
#endif

typedef int (*sym_calculateDataFileSizeAndDigest)( SignedDoc*, const char*, const char*, int );
typedef void (*sym_cleanupConfigStore)( ConfigurationStore* );
typedef int (*sym_convertStringToTimestamp)( SignedDoc*, const char*, Timestamp* );
typedef int (*sym_createOrReplacePrivateConfigItem)(ConfigurationStore*, const char*, const char* );
typedef int (*sym_createSignedDoc)( SignedDoc*, const char*, const char* );
typedef int (*sym_DataFile_delete)( SignedDoc*, const char* );
typedef int (*sym_DataFile_new)( DataFile**, SignedDoc*, const char*, const char*,
	const char*, const char*, long, const byte*, int, const char*, const char* );
typedef int (*sym_ddocPrepareSignature)( SignedDoc*, SignatureInfo**, const char*, const char*,
	const char*, const char*, const char*, X509*, const char* );
typedef int (*sym_ddocSaxReadSignedDocFromFile)( SignedDoc**, const char*, int, int );
typedef X509* (*sym_ddocSigInfo_GetOCSPRespondersCert)( const SignatureInfo * );
typedef DigiDocMemBuf* (*sym_ddocSigInfo_GetSignatureValue_Value)( SignatureInfo* );
typedef X509* (*sym_ddocSigInfo_GetSignersCert)( const SignatureInfo* );
typedef int (*sym_ddocSigInfo_SetSignatureValue)( SignatureInfo*, const char*, long );
typedef int (*sym_getCountOfDataFiles)( const SignedDoc* );
typedef int (*sym_getCountOfSignatures)( const SignedDoc* );
typedef DataFile* (*sym_getDataFile)( const SignedDoc*, int );
typedef char* (*sym_getErrorString)( int );
typedef SignatureInfo* (*sym_getSignature)( const SignedDoc*, int );
typedef void (*sym_finalizeDigiDocLib)();
typedef void (*sym_initDigiDocLib)();
typedef int (*sym_initConfigStore)( const char* );
typedef int (*sym_notarizeSignature)( SignedDoc*, SignatureInfo* );
typedef int (*sym_ddocSaxExtractDataFile)( SignedDoc*, const char*,
	const char*, const char*, const char* );
typedef int (*sym_SignatureInfo_delete)( SignedDoc*, const char* );
typedef int (*sym_SignedDoc_free)( SignedDoc* );
typedef int (*sym_SignedDoc_new)( SignedDoc**, const char*, const char* );
typedef int (*sym_verifySignatureAndNotary)( SignedDoc*, SignatureInfo*, const char* );

namespace digidoc
{

class DDocLibrary
{
public:
	static void destroy();
	static DDocLibrary *instance();

	bool ready;

	sym_calculateDataFileSizeAndDigest f_calculateDataFileSizeAndDigest;
	sym_cleanupConfigStore		f_cleanupConfigStore;
	sym_convertStringToTimestamp f_convertStringToTimestamp;
	sym_createOrReplacePrivateConfigItem f_createOrReplacePrivateConfigItem;
	sym_createSignedDoc			f_createSignedDoc;
	sym_DataFile_delete			f_DataFile_delete;
	sym_DataFile_new			f_DataFile_new;
	sym_ddocPrepareSignature	f_ddocPrepareSignature;
	sym_ddocSaxReadSignedDocFromFile f_ddocSaxReadSignedDocFromFile;
	sym_ddocSigInfo_GetOCSPRespondersCert f_ddocSigInfo_GetOCSPRespondersCert;
	sym_ddocSigInfo_GetSignatureValue_Value f_ddocSigInfo_GetSignatureValue_Value;
	sym_ddocSigInfo_GetSignersCert f_ddocSigInfo_GetSignersCert;
	sym_ddocSigInfo_SetSignatureValue f_ddocSigInfo_SetSignatureValue;
	sym_getCountOfDataFiles		f_getCountOfDataFiles;
	sym_getCountOfSignatures	f_getCountOfSignatures;
	sym_getDataFile				f_getDataFile;
	sym_getErrorString			f_getErrorString;
	sym_getSignature			f_getSignature;
	sym_finalizeDigiDocLib		f_finalizeDigiDocLib;
	sym_initDigiDocLib			f_initDigiDocLib;
	sym_initConfigStore			f_initConfigStore;
	sym_notarizeSignature		f_notarizeSignature;
	sym_ddocSaxExtractDataFile	f_ddocSaxExtractDataFile;
	sym_SignatureInfo_delete	f_SignatureInfo_delete;
	sym_SignedDoc_free			f_SignedDoc_free;
	sym_SignedDoc_new			f_SignedDoc_new;
	sym_verifySignatureAndNotary f_verifySignatureAndNotary;

private:
	DDocLibrary();
	~DDocLibrary();

	bool load();
	bool loadSymbols();
	void *resolve( const char *symbol );
	void unload();

	unsigned int ref;
	static DDocLibrary *m_instance;
#ifdef _WIN32
	HINSTANCE h;
#else
	void *h;
#endif
};

class SignatureDDOC;

class DDocPrivate
{
public:
	DDocPrivate();
	~DDocPrivate();

	DDocLibrary *lib;
	SignedDoc *doc;
	std::string filename;
	std::string tmpFolder;

	void loadSignatures();
	std::vector<SignatureDDOC*> signatures;
	std::vector<Document> documents;

	template<typename T>
	void throwCodeError( int err, const std::string &msg, int line ) const;
	void throwDocOpenError( int line ) const throw(BDocException);
	template<typename T>
	void throwError( const std::string &msg, int line, int err = -1,
		const digidoc::Exception::ExceptionCode &e = digidoc::Exception::NoException ) const;
	void throwSignError( const char *id, int err, const std::string &msg, int line ) const throw(BDocException);
};

class SignatureDDOCPrivate
{
public:
	DDocPrivate *priv;
	SignatureInfo *sig;
	DDocLibrary *lib;
};

}
