#include "Exception.h"

using namespace digidoc;

/**
 * @param file filename, where the exception was thrown.
 * @param line line of the file, where the exception was thrown.
 * @param msg error message.
 */
Exception::Exception(const std::string& file, int line, const std::string& msg)
 : file(file)
 , msg(msg)
 , line(line)
 , m_ddoc(-1)
 , m_code(NoException)
{}

/**
 * Convenience constructor when there is just one cause for this Exception.
 * 
 * @param file filename, where the exception was thrown.
 * @param line line of the file, where the exception was thrown.
 * @param msg error message.
 * @param cause cause of the exception.
 * @see hasCause()
 * @see getCause()
 */
Exception::Exception(const std::string& file, int line, const std::string& msg, const Exception& cause)
 : file(file)
 , msg(msg)
 , line(line)
 , m_ddoc(-1)
 , m_code(NoException)
{ addCause(cause); }

/**
 * @return return exception code
 */
Exception::ExceptionCode Exception::code() const { return m_code; }

/**
 * @return ddoc error code
 */
int Exception::ddoc() const { return m_ddoc; }

/**
 * @return ddoc error message
 */
std::string Exception::ddocMsg() const { return m_ddocMsg; }

/**
 * @return returns error message.
 */
std::string Exception::getMsg() const { return msg; }

void Exception::addCause(const Exception& cause) { causes.push_back(cause); }


/**
 * @return returns whether the exception has cause.
 */
bool Exception::hasCause() const { return !causes.empty(); }

/**
 * Returns exception causes (other exceptions that caused this exception).
 *
 * @return returns exception causes if the exception has cause otherwise empty collection.
 * @see hasCause()
 */
Exception::Causes Exception::getCauses() const { return causes; }

/**
 * @param code set exception code
 */
void Exception::setCode( ExceptionCode code ) { m_code = code; }

/**
 * @param err set ddoc error code
 */
void Exception::setDDoc( int err ) { m_ddoc = err; }

/**
 * @param msg set ddoc error message
 */
void Exception::setDDocMsg( const std::string &msg ) { m_ddocMsg = msg; }
